{% load i18n %}
{% load url from future %}

<div class="module cropduster-form nested-inline-form" id="{{ inline_admin_formset.formset.prefix }}-group" data-media-url="{{ media_url }}">

<input type="hidden" id="{{ final_attrs.id }}-0-id" name="{{ final_attrs.name }}-0-id" value="{{ final_attrs.value }}"/>
<input type="text"
       id="{{ final_attrs.id }}"
       name="{{ final_attrs.name }}"
       value="{{ file_value }}"
       class="cropduster-data-field cropduster-text-field"
       data-sizes="{{ sizes }}"
       data-upload-to="{{ upload_to }}"/>

<a href="#" class="cropduster-customfield cropduster-upload-form" data-cropduster-url="{% url 'cropduster-index' %}?pop=1">
    <div class="rounded-button">Upload Image</div>
    <div style="clear:both; height:3px"></div>
</a>

{% include "cropduster/inline.html" %}

<div class="manual_images cropduster-image-group">
    <div class="thumbs cropduster-images"></div>
</div>

<div style="clear:both"></div>


</div>